@@ -5,7 +5,7 @@ from typing import cast
 import numpy as np
 import pytest
 
-from array_api_extra._lib import Backend
+from array_api_extra._lib._backends import Backend
 from array_api_extra._lib._testing import xp_assert_close, xp_assert_equal
 from array_api_extra._lib._utils._compat import (
     array_namespace,
@@ -72,9 +72,10 @@ def test_assert_close_tolerance(xp: ModuleType):
 
 
 @param_assert_equal_close
-@pytest.mark.xfail_xp_backend(Backend.SPARSE, reason="index by sparse array")
+@pytest.mark.skip_xp_backend(Backend.SPARSE, reason="index by sparse array")
+@pytest.mark.skip_xp_backend(Backend.ARRAY_API_STRICTEST, reason="boolean indexing")
 def test_assert_close_equal_none_shape(xp: ModuleType, func: Callable[..., None]):  # type: ignore[explicit-any]
-    """On dask and other lazy backends, test that a shape with NaN's or None's
+    """On Dask and other lazy backends, test that a shape with NaN's or None's
     can be compared to a real shape.
     """
     a = xp.asarray([1, 2])
@@ -99,18 +100,18 @@ def test_assert_close_equal_none_shape(xp: ModuleType, func: Callable[..., None]
 
 
 def good_lazy(x: Array) -> Array:
-    """A function that behaves well in dask and jax.jit"""
+    """A function that behaves well in Dask and jax.jit"""
     return x * 2.0
 
 
 def non_materializable(x: Array) -> Array:
     """
     This function materializes the input array, so it will fail when wrapped in jax.jit
-    and it will trigger an expensive computation in dask.
+    and it will trigger an expensive computation in Dask.
     """
     xp = array_namespace(x)
     # Crashes inside jax.jit
-    # On dask, this triggers two computations of the whole graph
+    # On Dask, this triggers two computations of the whole graph
     if xp.any(x < 0.0) or xp.any(x > 10.0):
         msg = "Values must be in the [0, 10] range"
         raise ValueError(msg)
@@ -129,13 +130,18 @@ def non_materializable4(x: Array) -> Array:
     return non_materializable(x)
 
 
+def non_materializable5(x: Array) -> Array:
+    return non_materializable(x)
+
+
 lazy_xp_function(good_lazy)
 # Works on JAX and Dask
 lazy_xp_function(non_materializable2, jax_jit=False, allow_dask_compute=2)
+lazy_xp_function(non_materializable3, jax_jit=False, allow_dask_compute=True)
 # Works on JAX, but not Dask
-lazy_xp_function(non_materializable3, jax_jit=False, allow_dask_compute=1)
+lazy_xp_function(non_materializable4, jax_jit=False, allow_dask_compute=1)
 # Works neither on Dask nor JAX
-lazy_xp_function(non_materializable4)
+lazy_xp_function(non_materializable5)
 
 
 def test_lazy_xp_function(xp: ModuleType):
@@ -146,29 +152,30 @@ def test_lazy_xp_function(xp: ModuleType):
     xp_assert_equal(non_materializable(x), xp.asarray([1.0, 2.0]))
     # Wrapping explicitly disabled
     xp_assert_equal(non_materializable2(x), xp.asarray([1.0, 2.0]))
+    xp_assert_equal(non_materializable3(x), xp.asarray([1.0, 2.0]))
 
     if is_jax_namespace(xp):
-        xp_assert_equal(non_materializable3(x), xp.asarray([1.0, 2.0]))
+        xp_assert_equal(non_materializable4(x), xp.asarray([1.0, 2.0]))
         with pytest.raises(
             TypeError, match="Attempted boolean conversion of traced array"
         ):
-            _ = non_materializable4(x)  # Wrapped
+            _ = non_materializable5(x)  # Wrapped
 
     elif is_dask_namespace(xp):
         with pytest.raises(
             AssertionError,
             match=r"dask\.compute.* 2 times, but only up to 1 calls are allowed",
         ):
-            _ = non_materializable3(x)
+            _ = non_materializable4(x)
         with pytest.raises(
             AssertionError,
             match=r"dask\.compute.* 1 times, but no calls are allowed",
         ):
-            _ = non_materializable4(x)
+            _ = non_materializable5(x)
 
     else:
-        xp_assert_equal(non_materializable3(x), xp.asarray([1.0, 2.0]))
         xp_assert_equal(non_materializable4(x), xp.asarray([1.0, 2.0]))
+        xp_assert_equal(non_materializable5(x), xp.asarray([1.0, 2.0]))
 
 
 def static_params(x: Array, n: int, flag: bool = False) -> Array:
@@ -217,20 +224,21 @@ except ImportError:
     erf = None
 
 
-@pytest.mark.filterwarnings("ignore:__array_wrap__:DeprecationWarning")  # torch
+@pytest.mark.skip_xp_backend(Backend.TORCH_GPU, reason="device->host copy")
+@pytest.mark.filterwarnings("ignore:__array_wrap__:DeprecationWarning")  # PyTorch
 def test_lazy_xp_function_cython_ufuncs(xp: ModuleType, library: Backend):
     pytest.importorskip("scipy")
     assert erf is not None
     x = xp.asarray([6.0, 7.0])
-    if library in (Backend.ARRAY_API_STRICT, Backend.JAX):
-        # array-api-strict arrays are auto-converted to numpy
+    if library.like(Backend.ARRAY_API_STRICT, Backend.JAX):
+        # array-api-strict arrays are auto-converted to NumPy
         # which results in an assertion error for mismatched namespaces
-        # eager jax arrays are auto-converted to numpy in eager jax
+        # eager JAX arrays are auto-converted to NumPy in eager JAX
         # and fail in jax.jit (which lazy_xp_function tests here)
         with pytest.raises((TypeError, AssertionError)):
             xp_assert_equal(cast(Array, erf(x)), xp.asarray([1.0, 1.0]))
     else:
-        # cupy, dask and sparse define __array_ufunc__ and dispatch accordingly
+        # CuPy, Dask and sparse define __array_ufunc__ and dispatch accordingly
         # note that when sparse reduces to scalar it returns a np.generic, which
         # would make xp_assert_equal fail.
         xp_assert_equal(cast(Array, erf(x)), xp.asarray([1.0, 1.0]))
@@ -271,7 +279,7 @@ naked = ModuleType("naked")
 
 def f(x: Array) -> Array:
     xp = array_namespace(x)
-    # Crash in jax.jit and trigger compute() on dask
+    # Crash in jax.jit and trigger compute() on Dask
     if not xp.all(x):
         msg = "Values must be non-zero"
         raise ValueError(msg)
@@ -292,12 +300,12 @@ def test_lazy_xp_modules(xp: ModuleType, library: Backend):
     y = naked.f(x)
     xp_assert_equal(y, x)
 
-    if library is Backend.JAX:
+    if library.like(Backend.JAX):
         with pytest.raises(
             TypeError, match="Attempted boolean conversion of traced array"
         ):
             wrapped.f(x)
-    elif library is Backend.DASK:
+    elif library.like(Backend.DASK):
         with pytest.raises(AssertionError, match=r"dask\.compute"):
             wrapped.f(x)
     else:
